local _Questic = _Questic_ADDON_Globals;

_Questic.BackdropBorder = "Interface\\AddOns\\Questic\\Media\\BasicDynamicBorderTexture.blp";

local SuppressMsgCD = 0;
local function CheckQuesticEnable()
    if _Questic.GetVariable("Enable") then
        if _Questic.GetVariable("SuppressKeys") and ((_Questic.GetVariable("AltSuppress") and IsAltKeyDown())
                or (_Questic.GetVariable("ShiftSuppress") and IsShiftKeyDown())
                or (_Questic.GetVariable("CTRLSuppress") and IsControlKeyDown())) then
            if abs(SuppressMsgCD - GetTime()) > 0.1 then
                print("Questic: Addon suppressed by key modifier.");
                SuppressMsgCD = GetTime();
            end
            return false;
        end
        return true;
    end
    return false;
end

local function CheckQuesticQuestAccept(Type, Trivial)
    if CheckQuesticEnable() then
        if _Questic.GetVariable("AutoAccept")
                and ((Type == 0 and _Questic.GetVariable("AutoAcceptNormal"))
                or (Type == 2 and _Questic.GetVariable("AutoAcceptWeekly"))
                or (Type == 1 and _Questic.GetVariable("AutoAcceptDaily"))) then
            if not Trivial or (Trivial and _Questic.GetVariable("AutoAcceptLowLevel")) then
                return true;
            end
        end
    end
    return false;
end

local function TurnInQuestItems(ForceClose)
    if IsQuestCompletable() and not ForceClose then
        CompleteQuest()
    else
        CloseQuest()
    end
end

local function VerifyQuestTurnIn()
    if GetTitleText() == _Questic.GetActiveQuickQuest() then
        TurnInQuestItems()
    end
    if _Questic.GetVariable("AutoTurnin") then
        local SurpressAutoTurnIn = false
        local RequiredItems = {}
        local RequiredMoney = GetQuestMoneyToGet();
        if GetNumQuestItems() > 0 then
            for i = 1, GetNumQuestItems() do
                local Link = GetQuestItemLink("required", i)
                if Link then
                    if select(14, GetItemInfo(Link)) == 4 then --quest item
                        if not _Questic.GetVariable("AutoTurninQuestItems") then
                            SurpressAutoTurnIn = true
                        end
                        table.insert(RequiredItems, { ID = i, Type = "required", SubType = "item" })
                    else
                        if not _Questic.GetVariable("AutoTurninNonQuestItems") then
                            SurpressAutoTurnIn = true
                        end
                        table.insert(RequiredItems, { ID = i, Type = "required", SubType = "item" })
                    end
                end
            end
        end
        if GetNumQuestCurrencies() > 0 then
            if not _Questic.GetVariable("AutoturninCurrency") then
                SurpressAutoTurnIn = true
            end
            for i = 1, GetNumQuestCurrencies() do
                table.insert(RequiredItems, { ID = i, Type = "required", SubType = "currency" })
            end
        end
        if RequiredMoney > 0 then
            if not _Questic.GetVariable("AutoturninMoney") then
                SurpressAutoTurnIn = true
            end
            table.insert(RequiredItems, { ID = RequiredMoney, Type = "money", SubType = "money" })
        end

        if not SurpressAutoTurnIn then
            TurnInQuestItems()
        end
    end
end

local function SkipMovieFunc()
    if CheckQuesticEnable() and _Questic.GetVariable("CinematicSkip") then
        MovieFrame_OnMovieFinished(MovieFrame);
        print("Questic: Skipped movie!");
    end
end

hooksecurefunc(MovieFrame, "StartMovie", function(self, MovieID)
    SkipMovieFunc();
end)

----- Events
local FirstLoad = true;
_Questic:SetScript("OnEvent", function(self, Event, ...)
    if _Questic.GetVariable("Enable") then
        if Event ~= "MODIFIER_STATE_CHANGED" and Event ~= "QUEST_FINISHED" and Event ~= "QUEST_LOG_UPDATE" and Event ~= "GROUP_ROSTER_UPDATE" and Event ~= "CHAT_MSG_ADDON_LOGGED" and Event ~= "ZONE_CHANGED_NEW_AREA" and Event ~= "GOSSIP_CLOSED" and Event ~= "BAG_UPDATE" and Event ~= "BAG_UPDATE_COOLDOWN" then
            --Note: not quest finished to reduce quest suppress spam (especially as questic does nothing) (it is only registered to hide the item select frame if it was not used to select an item)
            if not CheckQuesticEnable() then
                return nil;
            end
        end

        if Event == "MODIFIER_STATE_CHANGED" then
            _Questic.GossipFunctions.GossipFrameSuppress();
            _Questic_ADDON_Globals.QuestFrameSuppress();
            _Questic.GossipQuickSelect:Update();

        elseif Event == "GOSSIP_SHOW" or Event == "QUEST_GREETING" then --Window to select availiable and completed quests
            local ActiveQuestTable, AvailableQuestTable = _Questic.FetchOfferedQuests();
            if _Questic.GetVariable("AutoTurnin") and #ActiveQuestTable > 0 then
                for _, Quest in ipairs(ActiveQuestTable) do
                    if Quest.Complete then
                        return _Questic.SelectQuest(Quest.Index, "Active", Quest.Type);
                    end
                end
            end
            if _Questic.GetVariable("AutoAccept") and #AvailableQuestTable > 0 then
                for _, Quest in ipairs(AvailableQuestTable) do
                    if _Questic.IsQuickQuest(Quest.Name) then
                        return nil
                    end
                end
                for _, Quest in ipairs(AvailableQuestTable) do
                    if CheckQuesticQuestAccept(Quest.Frequency, Quest.Trivial) then
                        return _Questic.SelectQuest(Quest.Index, "Available", Quest.Type);
                    end
                end
            end
            if _Questic.GetVariable("GossipAutoSelect") then
                local AutoGossipUsed = _Questic.GossipFunctions.AutoGossip();
                if AutoGossipUsed then
                    return nil;
                end
            end
            if Event == "GOSSIP_SHOW" then
                if _Questic.GetVariable("GossipQuickSelect") then
                    _Questic.GossipQuickSelect:Update();
                end
            else
                if _Questic.GetVariable("GossipQuickSelect") and _Questic.GetVariable("IncludeQuestsInGosseipSelect") then
                    _Questic.GossipQuickSelect:Update();
                end
            end
        elseif Event == "QUEST_PROGRESS" then --continue screen
            VerifyQuestTurnIn(); --for items requires to turn in or quest completed
        elseif Event == "QUEST_COMPLETE" then -- finished quest turn in
            if _Questic.GetVariable("AutoTurnin") then
                if GetNumQuestChoices() <= 1 then
                    GetQuestReward(1);
                end
            elseif GetTitleText() == _Questic.GetActiveQuickQuest() then
                _Questic.ClearActiveQuickQuest();
                if GetNumQuestChoices() <= 1 then
                    GetQuestReward(1);
                end
            end
        elseif Event == "QUEST_DETAIL" then --Quest info ready for accepting quest
            local QuestType = 0;
            if QuestIsDaily() then
                QuestType = 1;
            elseif QuestIsWeekly() then
                QuestType = 2;
            end
            if CheckQuesticQuestAccept(QuestType) then
                QuestDetailAcceptButton_OnClick();
            end
        elseif Event == "QUEST_ACCEPT_CONFIRM" then
            if CheckQuesticQuestAccept(1) then --normal quest type default
                ConfirmAcceptQuest();
            end
        elseif _Questic.GetVariable("CinematicSkip") and Event == "CINEMATIC_START" then
            CinematicFrame_CancelCinematic();
            print("Questic: Skipped cinematic!");
        elseif Event == "GROUP_ROSTER_UPDATE" or Event == "PLAYER_ENTERING_WORLD" then
            if FirstLoad then
                if _Questic.GetVariable("QuestItemBar") or (_Questic.GetVariable("AutoAcceptQuestsInBag") and _Questic.GetVariable("AutoAccept")) then
                    _Questic.BagScan();
                    if _Questic.GetVariable("QuestItemBar") then
                        _Questic.QuestItemBar.Update();
                    end
                end
                _Questic.CompileQuickQuestList();
                FirstLoad = false;
            end
            _Questic.UpdateQuestLog()
        elseif Event == "ZONE_CHANGED_NEW_AREA" or Event == "ZONE_CHANGED" or Event == "ZONE_CHANGED_INDOORS" then 
            _Questic.UpdateQuestLog()
        elseif Event == "QUEST_LOG_UPDATE" then
            _Questic.MinimapIcon:Refresh();
            _Questic.UpdateQuestLog()
        elseif Event == "GOSSIP_CLOSED" then
            _Questic.GossipQuickSelect:Update();
        elseif Event == "QUEST_ACCEPTED" and _Questic.GetVariable("AutoShare") then
            local QuestID = ...;
            _Questic.ShareQuest(QuestID);
        elseif Event == "BAG_UPDATE" and (_Questic.GetVariable("QuestItemBar") or (_Questic.GetVariable("AutoAcceptQuestsInBag") and _Questic.GetVariable("AutoAccept"))) then
            local BagID = ...;
            if BagID >= 0 and BagID <= 4 then
                _Questic.BagScan(BagID);
                if _Questic.GetVariable("QuestItemBar") then
                    _Questic.QuestItemBar.Update();
                end
            end
        elseif Event == "BAG_UPDATE_COOLDOWN" and _Questic.GetVariable("QuestItemBar") then
            _Questic.QuestItemBar.UpdateItemBarOverlay();
        end
    end

    if Event == "MODIFIER_STATE_CHANGED" or Event == "GOSSIP_SHOW" or Event == "GOSSIP_CLOSED" or Event == "QUEST_GREETING" or Event == "QUEST_PROGRESS" or Event == "QUEST_COMPLETE" or
            Event == "QUEST_DETAIL" or Event == "QUEST_ACCEPT_CONFIRM" or Event == "QUEST_FINISHED" or Event == "QUEST_ACCEPTED" or Event == "PLAYER_ENTERING_WORLD" then
        _Questic.GossipQuickSelect:Update();
    end
end)

_Questic:RegisterEvent("MODIFIER_STATE_CHANGED");
--Start Scanning for quests 
_Questic:RegisterEvent("GOSSIP_SHOW");
_Questic:RegisterEvent("GOSSIP_CLOSED");
_Questic:RegisterEvent("QUEST_GREETING");
--Turn in quest items 
_Questic:RegisterEvent("QUEST_PROGRESS");
--Complete quest 
_Questic:RegisterEvent("QUEST_COMPLETE");
--Accept quest 
_Questic:RegisterEvent("QUEST_DETAIL");
_Questic:RegisterEvent("QUEST_ACCEPT_CONFIRM"); --Escort start	 (player,questname)
--Finished Quests 
_Questic:RegisterEvent("QUEST_FINISHED");

--auto share
_Questic:RegisterEvent("QUEST_ACCEPTED");

--Cinematic Skip 
_Questic:RegisterEvent("CINEMATIC_START");

--group quest compare change 
--_Questic:RegisterEvent("GROUP_ROSTER_UPDATE");
_Questic:RegisterEvent("PLAYER_ENTERING_WORLD");

--update quest index refresh share menus (compare and quickshare dropdown)
_Questic:RegisterEvent("QUEST_LOG_UPDATE");
--_Questic:RegisterEvent("CHAT_MSG_ADDON_LOGGED");
_Questic:RegisterEvent("ZONE_CHANGED_NEW_AREA");
_Questic:RegisterEvent("ZONE_CHANGED");
_Questic:RegisterEvent("ZONE_CHANGED_INDOORS");
--bag update
_Questic:RegisterEvent("BAG_UPDATE");
_Questic:RegisterEvent("BAG_UPDATE_COOLDOWN");
C_ChatInfo.RegisterAddonMessagePrefix(_Questic.AddonComsPrefix);

hooksecurefunc(QuestFrame, "Show", function(self)
    _Questic_ADDON_Globals.QuestFrameSuppress();
end)

hooksecurefunc(QuestFrame, "Hide", function(self)
    _Questic.ClearActiveQuickQuest();
    _Questic_ADDON_Globals.QuestFrameSuppress();
end)

hooksecurefunc(GossipFrame, "Show", function(self)
    _Questic.GossipFunctions.GossipFrameSuppress();
    _Questic.GossipQuickSelect:Update(true);
end)

hooksecurefunc(GossipFrame, "Hide", function(self)
    _Questic.GossipFunctions.GossipFrameSuppress();
    _Questic.GossipQuickSelect:Update(false);
end) 


--[[
local OG = QuestInfoRewardItemCodeTemplate_OnClick
QuestInfoRewardItemCodeTemplate_OnClick = function(self,...)
    print("TEST",self.type,self:GetID())
    HandleModifiedItemClick(GetQuestItemLink(self.type, self:GetID()));

    --return OG(self,...)
end 
]]