local Widgets = _Avyr_GUI_Widgets_Lib
local MouseIsOver, insert = MouseIsOver, table.insert;
local BorderSize, Padding = Widgets.BorderSize, Widgets.Padding;

local Window = nil
local BtnOfOrigin = nil
local ColorSpectrumValues = {
    {1,0,0},
    {1,1,0},
    {0,1,0},
    {0,1,1},
    {0,0,1},
    {1,0,1},
    {1,0,0},
}
local r,g,b,a,darkness,lightness = 1,1,1,1,1
local inir,inig,inib,inia,inidarkness,inilightness = 1,1,1,1,1

local ConvTable = {
    [10] = "A",[11] = "B",[12] = "C",[13] = "D", [14] = "E", [15] = "F",
    ["0"] = 0,["1"] = 1, ["2"] = 2,["3"] = 3,["4"] = 4, ["5"] = 5,["6"] = 6,["7"] = 7, ["8"] = 8,["9"] = 9,
    ["A"] = 10, ["B"] = 11, ["C"] = 12, ["D"] = 13, ["E"] = 14, ["F"] = 15
}

local TranslationTable = {
    ["Base color"] = {
        ["deDE"] = "Basisfarbe",
    },
    ["Color select"] = {
        ["deDE"] = "Farbe auswahl",
    },
    ["Red"] = {
        ["deDE"] = "Rot",
    },
    ["Blue"] = {
        ["deDE"] = "Blau",
    },
    ["Green"] = {
        ["deDE"] = "grün",
    },
    ["Transparency"] = {
        ["deDE"] = "Transparenz",
    },
    ["Save changes"] = {
        ["deDE"] = "Änderungen speichern",
    },
    ["Undo changes"] = {
        ["deDE"] = "Änderungen verwerfen",
    },
}
local Translation = function(Text)
    if type(Text) == "string" then 
        if TranslationTable[Text] then 
            local Lang = GetLocale()
            if TranslationTable[Text][Lang] then 
                return TranslationTable[Text][Lang]
            else 
                return Text
            end 
        else 
            return Text.."NT"
        end 
    end 
end 

local colorShaderWitch = 4*15 + 4*20 + 7*Padding
local colorShaderWitchInternal = colorShaderWitch - 2*BorderSize
local colorDisplayWidth = colorShaderWitch - 20 - Padding

local function PrepareValuesForDisplay(Newr,Newg,Newb)
    if Newr > Newg then 
        if Newr > Newb then 
            darkness = Newr 
            if Newb > Newg then
                lightness = Newg
            else 
                lightness = Newb
            end 
        else 
            darkness = Newb 
            lightness = Newg
        end 
    else 
        if Newg > Newb then 
            if Newr > Newb then 
                lightness = Newb
            else 
                lightness = Newr
            end 
            darkness = Newg 
        else 
            darkness = Newb 
            lightness = Newr
        end 
    end 

    if darkness == 0 or lightness/darkness == 1 then --set red as the base color if its pure black or pure white
        r = 1
        g = 0
        b = 0
        lightness = 1
    else 
        lightness = lightness/darkness
        r = (Newr/darkness - lightness) / (1 - lightness)
        g = (Newg/darkness - lightness) / (1 - lightness)
        b = (Newb/darkness - lightness) / (1 - lightness)
    end 
end

local function HexNumConversion(Input)
    if type(Input) == "number" then 
        local value = Input*255 
        if value - floor(value) < 0.5 then 
            value = floor(value)
        else 
            value = ceil(value) 
        end 
        local bit2 = floor(value/16)
        local bit1 = value - bit2*16
        return (ConvTable[bit2] or bit2) .. "" .. (ConvTable[bit1] or bit1)
    elseif type(Input) == "string" then  
        local index = 1
        local colorByte = {0,0,0,0}
        for Bit in Input:gmatch(".") do 
            Bit = Bit:upper()
            if ConvTable[Bit] then 
                colorByte[ceil(index/2)] = colorByte[ceil(index/2)] + ConvTable[Bit] * 16^(2 - index + 2*(ceil(index/2) - 1))
                index = index + 1
                if not colorByte[ceil(index/2)] then 
                    break 
                end 
            end 
        end
        PrepareValuesForDisplay(colorByte[1]/255,colorByte[2]/255,colorByte[3]/255)
        a = colorByte[4]/255
    end 
end 

local function Round(number,dec)
    number = number * 10 ^ dec
    if number - floor(number) < 0.5 then 
        number = floor(number)
    else 
        number = ceil(number) 
    end 
    number = number / 10 ^ dec
    return number
end 


local function RefreshColorWindow(overwriteSP)
    local SP = 0
    if not overwriteSP then 
       -- print(r,g,b)
        if r == 1 then 
            if g > 0 then 
                SP = g/6
            elseif b > 0 then 
                SP =  5/6 + (1 - b)/6
            else 
                SP = 0
            end    
        elseif g == 1 then 
            if r > 0 then 
                SP = 1/6 + (1 - r)/6
            elseif b > 0 then 
                SP = 2/6 + b/6
                --print(SP)
            else 
                SP = 2/6
            end 
        elseif b == 1 then 
            if r > 0 then 
                SP = 4/6 + r/6
            elseif g > 0 then 
                SP = 3/6 + (1 - g)/6
            else 
                SP = 4/6
            end 
        end 
    else 
        SP = overwriteSP 
    end 
    Window.ColorSpectrumSliderBtn:SetPoint("CENTER",Window.ColorSpectrumSlider,"LEFT",BorderSize + SP*(Window.ColorSpectrumSlider:GetWidth() - 2*BorderSize),0)
    Window.ColorSpectrumSliderBtn:Backdrop("Backdrop",r,g,b,nil,nil,nil,1)

    Window.ColorShaderField:Backdrop("Backdrop",r,g,b,nil,nil,nil,1)

    Window.ColorShaderFieldPointer:SetPoint("CENTER",Window.ColorShaderField,"BOTTOMRIGHT",-lightness*colorShaderWitchInternal - BorderSize,darkness*colorShaderWitchInternal + BorderSize)

    local truer,trueg,trueb = (r + (1 - r) * lightness) * darkness,(g + (1 - g) * lightness) * darkness,(b + (1 - b) * lightness) * darkness

    Window.ColorShaderFieldPointer:Backdrop("Backdrop",truer,trueg,trueb,nil,nil,nil,1)

    Window.ColorDisplay:Backdrop("Backdrop",truer,trueg,trueb,nil,nil,nil,a)

    Window.ColorHexCode:SetValue(HexNumConversion(truer)..HexNumConversion(trueg)..HexNumConversion(trueb)..HexNumConversion(a))

    Window.RedComponentInput:SetValue(Round(truer,3))
    Window.RedComponentSliderBtn:Backdrop("Backdrop",truer,0,0,nil,nil,nil,truer)
    Window.RedComponentSliderBtn:SetPoint("CENTER",Window.RedComponentSlider,"LEFT",truer*(100 - 2*BorderSize) + BorderSize,0)

    Window.GreenComponentInput:SetValue(Round(trueg,3))
    Window.GreenComponentSliderBtn:Backdrop("Backdrop",0,trueg,0,nil,nil,nil,trueg)
    Window.GreenComponentSliderBtn:SetPoint("CENTER",Window.GreenComponentSlider,"LEFT",trueg*(100 - 2*BorderSize) + BorderSize,0)

    Window.BlueComponentInput:SetValue(Round(trueb,3))
    Window.BlueComponentSliderBtn:Backdrop("Backdrop",0,0,trueb,nil,nil,nil,trueb)
    Window.BlueComponentSliderBtn:SetPoint("CENTER",Window.BlueComponentSlider,"LEFT",trueb*(100 - 2*BorderSize) + BorderSize,0)

    Window.AlphaComponentInput:SetValue(Round(a,3))
    Window.AlphaComponentSliderBtn:Backdrop("Backdrop",1,1,1,nil,nil,nil,a)
    Window.AlphaComponentSliderBtn:SetPoint("CENTER",Window.AlphaComponentSlider,"LEFT",a*(100 - 2*BorderSize) + BorderSize,0)
end 
-- actualr = (baser + (1 - baser) * lightness) * darkness
-- actualr = (baser + lightness - baser * lightness) * darkness
-- actualr = (baser * (1 - lightness) + lightness) * darkness
-- actualr/darkness = baser * (1 - lightness) + lightness
-- actualr/darkness - lightness = baser * (1 - lightness)
-- (actualr/darkness - lightness) / (1 - lightness) = baser
-- baser = (actualr/darkness - lightness) / (1 - lightness)

local function ShaderMoveFunc()
    local MouseX, MouseY = GetCursorPosition()
    local Scale = UIParent:GetEffectiveScale() * Window.ColorShaderField:GetScale()
    MouseX = MouseX / Scale
    MouseY = MouseY / Scale
    
    darkness = (MouseY - Window.ColorShaderField:GetBottom() - BorderSize)/(Window.ColorShaderField:GetHeight() - 2*BorderSize)
    lightness = 1 - (MouseX - Window.ColorShaderField:GetLeft() - BorderSize)/(Window.ColorShaderField:GetWidth() - 2*BorderSize)

    if darkness > 1 then darkness = 1 elseif darkness < 0 then darkness = 0 end 
    if lightness > 1 then lightness = 1 elseif lightness < 0 then lightness = 0 end

    RefreshColorWindow()
end 

local function ColSpectrumSliderFunc()
    local MouseX, MouseY = GetCursorPosition()
    local Scale = UIParent:GetEffectiveScale() * Window.ColorSpectrumSlider:GetScale()
    MouseX = MouseX / Scale
    MouseY = MouseY / Scale
    local SP = (MouseX - Window.ColorSpectrumSlider:GetLeft() - BorderSize)/(Window.ColorSpectrumSlider:GetWidth() - 2*BorderSize)
    if SP > 1 then SP = 1 elseif SP < 0 then SP = 0 end  
    local ColIndex = floor(SP * 6)
    local ColRatio = SP * 6  - ColIndex
    ColIndex = ColIndex + 1
    if ColIndex == 7 then 
        r = ColorSpectrumValues[7][1]
        g = ColorSpectrumValues[7][2]
        b = ColorSpectrumValues[7][3]
    else 
        r = ColorSpectrumValues[ColIndex][1] - ColRatio*(ColorSpectrumValues[ColIndex][1] - ColorSpectrumValues[ColIndex + 1][1])
        g = ColorSpectrumValues[ColIndex][2] - ColRatio*(ColorSpectrumValues[ColIndex][2] - ColorSpectrumValues[ColIndex + 1][2])
        b = ColorSpectrumValues[ColIndex][3] - ColRatio*(ColorSpectrumValues[ColIndex][3] - ColorSpectrumValues[ColIndex + 1][3])
    end
    RefreshColorWindow(SP)
end  

local function UpdateColorComponent(Color,Value)
    if Value > 1 then Value = 1 elseif Value < 0 then Value = 0 end 
    if Color == "r" then 
        local truer = (r + (1 - r) * lightness) * darkness
        if truer == Value then return nil end 
        local trueg,trueb = (g + (1 - g) * lightness) * darkness,(b + (1 - b) * lightness) * darkness
        truer = Value
        PrepareValuesForDisplay(truer,trueg,trueb)
    elseif Color == "g" then 
        local trueg = (g + (1 - g) * lightness) * darkness
        if trueg == Value then return nil end 
        local truer,trueb = (r + (1 - r) * lightness) * darkness,(b + (1 - b) * lightness) * darkness
        trueg = Value
        PrepareValuesForDisplay(truer,trueg,trueb)
    elseif Color == "b" then 
        local trueb = (b + (1 - b) * lightness) * darkness
        if trueb == Value then return nil end 
        local truer,trueg = (r + (1 - r) * lightness) * darkness,(g + (1 - g) * lightness) * darkness
        trueb = Value
        PrepareValuesForDisplay(truer,trueg,trueb)
    elseif Color == "a" then 
        a = Value
    end 
    RefreshColorWindow()
end 

local function UpdateColorSliderFunc(self)
    local MouseX, MouseY = GetCursorPosition()
    local Scale = UIParent:GetEffectiveScale() * self:GetScale()
    MouseX = MouseX / Scale
    MouseY = MouseY / Scale
    local SP = (MouseX - self:GetLeft() - BorderSize)/(self:GetWidth() - 2*BorderSize)
    if SP > 1 then SP = 1 elseif SP < 0 then SP = 0 end  
    UpdateColorComponent(self.colVarName,SP)
end 

local function CallColorSelect(BoO,Newr,Newg,Newb,Newa)
    if Window and BtnOfOrigin and BtnOfOrigin == BoO then 
        Window:Release()
        Window = nil
        BtnOfOrigin = nil
    else 
        PrepareValuesForDisplay(Newr,Newg,Newb)
        a = Newa
        inir,inig,inib,inia,inidarkness,inilightness = r,g,b,a,darkness,lightness
        BtnOfOrigin = BoO
        if Window then 
            RefreshColorWindow()
        else 
            Window = Widgets:CreateWindow(UIParent,Translation("Color select"))
            Window:SetFrameStrata("HIGH")
            Window:Size(colorDisplayWidth + colorShaderWitch + 60 + 100 + 2*BorderSize + 10*Padding,100)
            Window.OnCloseFunc = function()
                Window = nil
                BtnOfOrigin = nil
            end 
            local Height = Window.Header:GetHeight() + 2*Padding + 3*BorderSize
            local WO = Padding + BorderSize
            local SpectrumWidth = Window:GetWidth() - 2*Padding - 2*BorderSize - 20
            local IntSW = (SpectrumWidth - 2*BorderSize)/6

            --ColorDisplay
            Window.ColorDisplay = Widgets:CreateFrame(Window)
            Window.ColorDisplay:SetPoint("TOPLEFT",WO,-Height)
            Window.ColorDisplay:Size(colorDisplayWidth,colorDisplayWidth)
            Window.ColorDisplay:Backdrop("Border")

            Window.ColorHexCode = Widgets:CreateBasicTextBox(Window, false, "CENTER", "CENTER")
            Window.ColorHexCode:Size(colorDisplayWidth,20)
            Window.ColorHexCode:SetPoint("TOPLEFT",Window.ColorDisplay,"BOTTOMLEFT",0,-Padding) 
            Window.ColorHexCode.OnValueChanged = function(self,value)
                HexNumConversion(value)
                RefreshColorWindow()
            end 

            Window.ColorDisplay["ColorBackdropSectionMain"] = Widgets:FetchTexture(Window.ColorDisplay, "BACKGROUND",-2)
            Window.ColorDisplay["ColorBackdropSectionMain"]:SetPoint("TOPLEFT")
            Window.ColorDisplay["ColorBackdropSectionMain"]:SetSize(colorDisplayWidth,colorDisplayWidth)
            Window.ColorDisplay["ColorBackdropSectionMain"]:SetColorTexture(1,1,1,1)
            for row = 1,5 do 
                for item = 1,2 + row % 2 do 
                end 
            end 
            for subSection = 1,12 do 
                local grouping = 5*(ceil(subSection/5) - 1) 
                local subGrouping = ceil((subSection - grouping)/2.5) - 1
                local Pos = 0
                if subGrouping == 0 then 
                    Pos = 1 + 2*(subSection - grouping - 1)
                else 
                    Pos = 2*(subSection - grouping - 3)
                end 
                Window.ColorDisplay["ColorBackdropSection"..subSection] = Widgets:FetchTexture(Window.ColorDisplay, "BACKGROUND",-1)
                Window.ColorDisplay["ColorBackdropSection"..subSection]:SetPoint("TOPLEFT",Pos*colorDisplayWidth/5,-(ceil(subSection/2.5) - 1)*colorDisplayWidth/5)
                Window.ColorDisplay["ColorBackdropSection"..subSection]:SetSize(colorDisplayWidth/5,colorDisplayWidth/5)
                Window.ColorDisplay["ColorBackdropSection"..subSection]:SetColorTexture(0,0,0,1)

            end


            --[[for subSection = 1,25 do 
                Window.ColorDisplay["ColorBackdropSection"..subSection] = Widgets:FetchTexture(Window.ColorDisplay, "BACKGROUND",-1)
                Window.ColorDisplay["ColorBackdropSection"..subSection]:SetPoint("TOPLEFT",(subSection - 5*(ceil(subSection/5) - 1) - 1)*colorDisplayWidth/5,-(ceil(subSection/5) - 1)*colorDisplayWidth/5)
                Window.ColorDisplay["ColorBackdropSection"..subSection]:SetSize(colorDisplayWidth/5,colorDisplayWidth/5)
                Window.ColorDisplay["ColorBackdropSection"..subSection]:SetColorTexture(2*ceil(subSection/2) - subSection,2*ceil(subSection/2) - subSection,2*ceil(subSection/2) - subSection,1)
            end ]]

            --Shader
            Window.ColorShaderField = Widgets:CreateFrame(Window)
            Window.ColorShaderField:SetPoint("TOPLEFT",Window.ColorDisplay,"TOPRIGHT",3*Padding,0)
            Window.ColorShaderField:Size(colorShaderWitch,colorShaderWitch)
            Window.ColorShaderField:Backdrop("Border")

            Window["ColorShaderFieldSection"..1] = Widgets:FetchTexture(Window.ColorShaderField,nil,1)
            Window["ColorShaderFieldSection"..1]:SetPoint("LEFT",Window.ColorShaderField,"LEFT",BorderSize,0)
            Window["ColorShaderFieldSection"..1]:SetSize(colorShaderWitchInternal,colorShaderWitchInternal)
            Window["ColorShaderFieldSection"..1]:SetColorTexture(1,1,1)
            Window["ColorShaderFieldSection"..1]:SetGradientAlpha("HORIZONTAL",1,1,1,1,1,1,1,0)

            Window["ColorShaderFieldSection"..2] = Widgets:FetchTexture(Window.ColorShaderField,nil,2)
            Window["ColorShaderFieldSection"..2]:SetPoint("LEFT",Window.ColorShaderField,"LEFT",BorderSize,0)
            Window["ColorShaderFieldSection"..2]:SetSize(colorShaderWitchInternal,colorShaderWitchInternal)
            Window["ColorShaderFieldSection"..2]:SetColorTexture(0,0,0)
            Window["ColorShaderFieldSection"..2]:SetGradientAlpha("VERTICAL",0,0,0,1,0,0,0,0)

            Window.ColorShaderField.MouseInteraction = ShaderMoveFunc
            Window.ColorShaderField:MakeInteractable(true);

            Window.ColorShaderFieldPointer = Widgets:CreateFrame(Window.ColorShaderField)
            Window.ColorShaderFieldPointer:Size(15,15)
            Window.ColorShaderFieldPointer:Backdrop("Border")

            Window.ColorShaderFieldPointer.MouseDownFeedback = ShaderMoveFunc
            Window.ColorShaderFieldPointer:MakeInteractable(true);

            --Color components
            --red
            Window.RedComponentInputText = Widgets:FetchFontString(Window)
            Window.RedComponentInputText:SetPoint("TOPLEFT",Window.ColorShaderField,"TOPRIGHT",4*Padding,0)
            Window.RedComponentInputText:SetText(Translation("Red"))
            Window.RedComponentInputText:SetHeight(15)
            Window.RedComponentInput = Widgets:CreateBasicTextBox(Window, false, "LEFT", "CENTER")
            Window.RedComponentInput:Size(60,20)
            Window.RedComponentInput:SetPoint("TOPLEFT",Window.RedComponentInputText,"BOTTOMLEFT",-Padding,-Padding)  
            Window.RedComponentInput.NumericInputFilter = true
            Window.RedComponentInput.OnValueChanged = function(self,value)
                local IntValue = tonumber(value)
                if IntValue then 
                    UpdateColorComponent("r",IntValue)
                end 
            end 

            Window.RedComponentSlider = Widgets:CreateFrame(Window)
            Window.RedComponentSlider:SetPoint("TOPLEFT",Window.RedComponentInput,"TOPRIGHT",2*Padding,0)
            Window.RedComponentSlider:Backdrop("Border")
            Window.RedComponentSlider:Backdrop("Backdrop", 1, 0, 0, 1, 0, 0, 0, 1)
            Window.RedComponentSlider:Size(100,20)

            Window.RedComponentSlider.colVarName = "r"
            Window.RedComponentSlider.MouseInteraction = UpdateColorSliderFunc
            Window.RedComponentSlider.MouseDownFeedback = UpdateColorSliderFunc
            Window.RedComponentSlider:MakeInteractable(true);

            Window.RedComponentSliderBtn = Widgets:CreateFrame(Window.RedComponentSlider)
            Window.RedComponentSliderBtn:Backdrop("Border")
            Window.RedComponentSliderBtn:Size(10,20 + 2*Padding)

            --green
            Window.GreenComponentInputText = Widgets:FetchFontString(Window)
            Window.GreenComponentInputText:SetPoint("TOPLEFT",Window.RedComponentInput,"BOTTOMLEFT",Padding,-Padding)
            Window.GreenComponentInputText:SetText(Translation("Green"))
            Window.GreenComponentInputText:SetHeight(15)
            Window.GreenComponentInput = Widgets:CreateBasicTextBox(Window, false, "LEFT", "CENTER")
            Window.GreenComponentInput:Size(60,20)
            Window.GreenComponentInput:SetPoint("TOPLEFT",Window.GreenComponentInputText,"BOTTOMLEFT",-Padding,-Padding)  
            Window.GreenComponentInput.NumericInputFilter = true
            Window.GreenComponentInput.OnValueChanged = function(self,value)
                local IntValue = tonumber(value)
                if IntValue then 
                    UpdateColorComponent("g",IntValue)
                end 
            end 

            Window.GreenComponentSlider = Widgets:CreateFrame(Window)
            Window.GreenComponentSlider:SetPoint("TOPLEFT",Window.GreenComponentInput,"TOPRIGHT",2*Padding,0)
            Window.GreenComponentSlider:Backdrop("Border")
            Window.GreenComponentSlider:Backdrop("Backdrop", 0, 1, 0, 0, 1, 0, 0, 1)
            Window.GreenComponentSlider:Size(100,20)

            Window.GreenComponentSlider.colVarName = "g"
            Window.GreenComponentSlider.MouseInteraction = UpdateColorSliderFunc
            Window.GreenComponentSlider.MouseDownFeedback = UpdateColorSliderFunc
            Window.GreenComponentSlider:MakeInteractable(true);

            Window.GreenComponentSliderBtn = Widgets:CreateFrame(Window.GreenComponentSlider)
            Window.GreenComponentSliderBtn:Backdrop("Border")
            Window.GreenComponentSliderBtn:Size(10,20 + 2*Padding)

            --blue
            Window.BlueComponentInputText = Widgets:FetchFontString(Window)
            Window.BlueComponentInputText:SetPoint("TOPLEFT",Window.GreenComponentInput,"BOTTOMLEFT",Padding,-Padding)
            Window.BlueComponentInputText:SetText(Translation("Blue"))
            Window.BlueComponentInputText:SetHeight(15)
            Window.BlueComponentInput = Widgets:CreateBasicTextBox(Window, false, "LEFT", "CENTER")
            Window.BlueComponentInput:Size(60,20)
            Window.BlueComponentInput:SetPoint("TOPLEFT",Window.BlueComponentInputText,"BOTTOMLEFT",-Padding,-Padding)  
            Window.BlueComponentInput.NumericInputFilter = true
            Window.BlueComponentInput.OnValueChanged = function(self,value)
                local IntValue = tonumber(value)
                if IntValue then 
                    UpdateColorComponent("b",IntValue)
                end 
            end 

            Window.BlueComponentSlider = Widgets:CreateFrame(Window)
            Window.BlueComponentSlider:SetPoint("TOPLEFT",Window.BlueComponentInput,"TOPRIGHT",2*Padding,0)
            Window.BlueComponentSlider:Backdrop("Border")
            Window.BlueComponentSlider:Backdrop("Backdrop", 0, 0, 1, 0, 0, 1, 0, 1)
            Window.BlueComponentSlider:Size(100,20)

            Window.BlueComponentSlider.colVarName = "b"
            Window.BlueComponentSlider.MouseInteraction = UpdateColorSliderFunc
            Window.BlueComponentSlider.MouseDownFeedback = UpdateColorSliderFunc
            Window.BlueComponentSlider:MakeInteractable(true);

            Window.BlueComponentSliderBtn = Widgets:CreateFrame(Window.BlueComponentSlider)
            Window.BlueComponentSliderBtn:Backdrop("Border")
            Window.BlueComponentSliderBtn:Size(10,20 + 2*Padding)

            --alpha
            Window.AlphaComponentInputText = Widgets:FetchFontString(Window)
            Window.AlphaComponentInputText:SetPoint("TOPLEFT",Window.BlueComponentInput,"BOTTOMLEFT",Padding,-Padding)
            Window.AlphaComponentInputText:SetText(Translation("Transparency"))
            Window.AlphaComponentInputText:SetHeight(15)
            Window.AlphaComponentInput = Widgets:CreateBasicTextBox(Window, false, "LEFT", "CENTER")
            Window.AlphaComponentInput:Size(60,20)
            Window.AlphaComponentInput:SetPoint("TOPLEFT",Window.AlphaComponentInputText,"BOTTOMLEFT",-Padding,-Padding)  
            Window.AlphaComponentInput.NumericInputFilter = true
            Window.AlphaComponentInput.OnValueChanged = function(self,value)
                local IntValue = tonumber(value)
                if IntValue then 
                    UpdateColorComponent("a",IntValue)
                end 
            end 

            Window.AlphaComponentSlider = Widgets:CreateFrame(Window)
            Window.AlphaComponentSlider:SetPoint("TOPLEFT",Window.AlphaComponentInput,"TOPRIGHT",2*Padding,0)
            Window.AlphaComponentSlider:Backdrop("Border")
            Window.AlphaComponentSlider:Backdrop("Backdrop", 1, 1, 1, 1, 1, 1, 0, 1)
            Window.AlphaComponentSlider:Size(100,20)

            Window.AlphaComponentSlider.colVarName = "a"
            Window.AlphaComponentSlider.MouseInteraction = UpdateColorSliderFunc
            Window.AlphaComponentSlider.MouseDownFeedback = UpdateColorSliderFunc
            Window.AlphaComponentSlider:MakeInteractable(true);

            Window.AlphaComponentSliderBtn = Widgets:CreateFrame(Window.AlphaComponentSlider)
            Window.AlphaComponentSliderBtn:Backdrop("Border")
            Window.AlphaComponentSliderBtn:Size(10,20 + 2*Padding)



            Height = Height + colorShaderWitch + Padding

            Window.ColorSpectrumSliderTitle = Widgets:FetchFontString(Window)
            Window.ColorSpectrumSliderTitle:SetPoint("TOPLEFT",Padding + WO,-Height)
            Window.ColorSpectrumSliderTitle:SetText(Translation("Base color"))

            Height = Height + Padding + 2*BorderSize + Window.ColorSpectrumSliderTitle:GetHeight()

            Window.ColorSpectrumSlider = Widgets:CreateFrame(Window)
            Window.ColorSpectrumSlider:SetPoint("TOPLEFT",WO + 10,-Height)
            Window.ColorSpectrumSlider:Backdrop()
            Window.ColorSpectrumSlider:Size(SpectrumWidth,30)

            for i = 1,6 do 
                Window["ColorSpectrumSliderSection"..i] = Widgets:FetchTexture(Window.ColorSpectrumSlider)
                Window["ColorSpectrumSliderSection"..i]:SetPoint("LEFT",Window.ColorSpectrumSlider,"LEFT",BorderSize + (i - 1)*IntSW,0)
                Window["ColorSpectrumSliderSection"..i]:SetSize(IntSW,30 - 2*BorderSize)
                Window["ColorSpectrumSliderSection"..i]:SetColorTexture(1,1,1)
                Window["ColorSpectrumSliderSection"..i]:SetGradient("HORIZONTAL", 
                    ColorSpectrumValues[i][1],ColorSpectrumValues[i][2],ColorSpectrumValues[i][3],
                    ColorSpectrumValues[i + 1][1],ColorSpectrumValues[i + 1][2],ColorSpectrumValues[i + 1][3])
            end 

            Height = Height + Window.ColorSpectrumSlider:GetHeight() + 2*BorderSize + Padding

            Window.ColorSpectrumSlider.MouseInteraction = ColSpectrumSliderFunc
            Window.ColorSpectrumSlider.MouseDownFeedback = ColSpectrumSliderFunc
            Window.ColorSpectrumSlider:MakeInteractable(true);
            
            Window.ColorSpectrumSliderBtn = Widgets:CreateFrame(Window.ColorSpectrumSlider)
            Window.ColorSpectrumSliderBtn:Backdrop("Border")
            Window.ColorSpectrumSliderBtn:Size(15,30 + 4*BorderSize)
            

            

            Window.SaveBtn = Widgets:CreateButton(Window)
            Window.SaveBtn:Size((Window:GetWidth() - 3*Padding - 2*BorderSize)/2,20)
            Window.SaveBtn:SetPoint("TOPLEFT",WO,-Height)
            Window.SaveBtn:SetText(Translation("Save changes"))
            Window.SaveBtn.OnClick = function()
                local cr, cg, cb = (r + (1 - r) * lightness) * darkness, (g + (1 - g) * lightness) * darkness, (b + (1 - b) * lightness) * darkness
                BtnOfOrigin:SetValue(cr,cg,cb,a,true)
                CallColorSelect(BtnOfOrigin)
            end 

            WO = WO + (Window:GetWidth() - 3*Padding - 2*BorderSize)/2 + Padding

            Window.UndoBtn = Widgets:CreateButton(Window)
            Window.UndoBtn:Size((Window:GetWidth() - 3*Padding - 2*BorderSize)/2,20)
            Window.UndoBtn:SetPoint("TOPLEFT",WO,-Height)
            Window.UndoBtn:SetText(Translation("Undo changes"))
            Window.UndoBtn.OnClick = function()
                r,g,b,a,darkness,lightness = inir,inig,inib,inia,inidarkness,inilightness
                RefreshColorWindow()
            end 

            Height = Height + 20 + Padding

            Height = Height + BorderSize

            Window:Size(nil,Height)

            RefreshColorWindow()
        end 
    end 
end 

Widgets.CreateColorSelect = function(self, Parent)
    local SelectButton = Widgets:CreateFrame(Parent)

    SelectButton.ColorBtn = Widgets:CreateFrame(SelectButton)
    SelectButton.ColorBtn:SetPoint("TOPLEFT")
    SelectButton.ColorBtn:SetPoint("BOTTOMLEFT")
    SelectButton.ColorBtn:Backdrop()
    SelectButton.ColorBtn:Size(20)

    SelectButton:SizeChanged(function()
        SelectButton.ColorBtn:Size(SelectButton:GetHeight())
    end)


    SelectButton.Colr = 0
    SelectButton.Colg = 1
    SelectButton.Colb = 0
    SelectButton.Cola = 1

    SelectButton.SetValue = function(self,r,g,b,a,userInput)
        SelectButton.Colr = r or SelectButton.Colr
        SelectButton.Colg = g or SelectButton.Colg
        SelectButton.Colb = b or SelectButton.Colb
        SelectButton.Cola = a or SelectButton.Cola
        SelectButton.ColorBtn:Backdrop("Backdrop",SelectButton.Colr,SelectButton.Colg,SelectButton.Colb,nil,nil,nil,SelectButton.Cola)
        if userInput and self.OnValueChanged then 
            self.OnValueChanged(self,SelectButton.Colr,SelectButton.Colg,SelectButton.Colb,SelectButton.Cola)
        end 
    end

    SelectButton:SetText("", 11, 1, 1, 1, "LEFT")
    SelectButton.Text1:SetPoint("BOTTOMLEFT", BorderSize + Padding + SelectButton.ColorBtn:GetWidth(), BorderSize + Padding)

    SelectButton.State = false;
    
    SelectButton.EnterLeave = function(self, Enter)
        if Enter then
            if self.State ~= "Pushed" and not self.Disable then
                self.State = "MouseOver"
                self.ColorBtn:PresetBorders(self.State)
            end
        else
            if self.State ~= "Pushed" and not self.Disable then
                self.State = "Normal"
                self.ColorBtn:PresetBorders(self.State)
            end
        end
    end

    SelectButton.MouseInteraction = function(self, Up)
        if Up then
            if not self.Disable then
                if MouseIsOver(self) then
                    self.State = "MouseOver"
                    self.ColorBtn:PresetBorders(self.State)
                else
                    self.ColorBtn:Backdrop("Border")
                    self.State = "Normal"
                    self.ColorBtn:PresetBorders(self.State)
                end
            end
        else
            if not self.Disable then
                self.State = "Pushed"
                self.ColorBtn:PresetBorders(self.State)
                CallColorSelect(self,self.Colr,self.Colg,self.Colb,self.Cola)  
            end
        end
    end
    SelectButton:MakeInteractable(true);
    

    return SelectButton
end
