local Widgets = _Avyr_GUI_Widgets_Lib
local pairs, ipairs, find, type, collectgarbage, insert, getn, remove = pairs, ipairs, string.find, type, collectgarbage, table.insert, getn, table.remove;
local wipe = table.wipe
local BorderSize, Padding = Widgets.BorderSize, Widgets.Padding;

Widgets.ContainerTableDirectory = {};

local ScrollMulti = 4

local function ReleaseElements(self)
    local i = 1;
    for _ = 1, getn(self.LM) do
        if not self.LM[i] then
            break;
        end
        if find(self.LM[i], "Container_Element_") then
            self[self.LM[i]]:Release()
            self[self.LM[i]] = nil
            remove(self.LM, i);
        else
            i = i + 1;
        end
    end
end

local function AddItems(self, Items)
    ReleaseElements(self)
    local Height = Padding
    local i = 1
    local PrevElement = ""
    local PrevIndent = 0
    for _, v in ipairs(Items) do
        --Types = LB, TB, SB, CB, NB, DB, DB_BL, PB, SL
        if v.Type == "DB" or v.Type == "DB_BL" then
            self["Container_Element_" .. (v.Name or i)] = Widgets:CreateButton(self.Content)
            self["Container_Element_" .. (v.Name or i)]:Size(self.Content:GetWidth() - 2 * Padding - (v.HSpacing or 0), 20 + (((v.Lines or 1) - 1) * 12))
            if type(v.Text) == "table" then 
                self["Container_Element_" .. (v.Name or i)]:SetText(v.Text or "nil")
            else 
                self["Container_Element_" .. (v.Name or i)]:SetText(v.Text or "nil", 11, 1, 1, 1, v.Alignment)
            end 
            if v.Icon then
                local l, r, t, b, rC, gC, bC, BorderGap, Rotation;
                if v.IconData then
                    l, r, t, b, rC, gC, bC, BorderGap, Rotation = unpack(v.IconData)
                end
                self["Container_Element_" .. (v.Name or i)]:SetIcon(v.Icon ,l, r, t, b, rC, gC, bC, BorderGap, Rotation, "LEFT")
                if self["Container_Element_" .. (v.Name or i)].Text1 then
                    self["Container_Element_" .. (v.Name or i)].Text1:SetPoint("BOTTOMLEFT",self["Container_Element_" .. (v.Name or i)].Icon,"BOTTOMRIGHT", Padding, Padding)
                end
            end
            self["Container_Element_" .. (v.Name or i)]:SetFocus(v.Selected)
            self["Container_Element_" .. (v.Name or i)]:SetDisable(v.Disable)
            if v.Type == "DB_BL" then
                self["Container_Element_" .. (v.Name or i)]:SetBorderless(true);
            end
            self["Container_Element_" .. (v.Name or i)].OnClick = v.Func
        elseif v.Type == "LB" then
            self["Container_Element_" .. (v.Name or i)] = Widgets:FetchFontString(self.Content)
            self["Container_Element_" .. (v.Name or i)]:SetText(v.Text)
            self["Container_Element_" .. (v.Name or i)]:SetWidth(self.Content:GetWidth() - 2 * Padding - (v.HSpacing or 0))

            self["Container_Element_" .. (v.Name or i)]:SetTextColor(1, 0.82, 0)
            local Font, _, Flags = self["Container_Element_" .. (v.Name or i)]:GetFont()
            self["Container_Element_" .. (v.Name or i)]:SetFont(Font, 11, Flags)
            self["Container_Element_" .. (v.Name or i)]:SetJustifyH(v.Alignment or "CENTER")
        elseif v.Type == "CB" then
            self["Container_Element_" .. (v.Name or i)] = Widgets:CreateCheckbox(self.Content)
            self["Container_Element_" .. (v.Name or i)]:SetText(v.Text, 11, 1, 1, 1, "LEFT", "TOP")
            self["Container_Element_" .. (v.Name or i)]:Size(self.Content:GetWidth() - 2 * Padding - (v.HSpacing or 0), 20 + (((v.Lines or 1) - 1) * 12))
            self["Container_Element_" .. (v.Name or i)]:SetChecked(v.Value)
            self["Container_Element_" .. (v.Name or i)]:SetDisable(v.Disable)
            self["Container_Element_" .. (v.Name or i)].OnChecked = v.Func
        elseif v.Type == "SB" then
            self["Container_Element_" .. (v.Name or i)] = Widgets:CreateSelectbutton(self.Content)
            self["Container_Element_" .. (v.Name or i)]:Size(self.Content:GetWidth() - 2 * Padding - (v.HSpacing or 0), 40)
            self["Container_Element_" .. (v.Name or i)]:SetText(v.Text, 11, 1, 1, 1, "LEFT", "TOP")
            if type(v.Options) == "function" then
                self["Container_Element_" .. (v.Name or i)].SelectListFunc = v.Options
            else
                self["Container_Element_" .. (v.Name or i)].SelectList = v.Options
            end
            self["Container_Element_" .. (v.Name or i)]:SetSelected(v.Value)
            self["Container_Element_" .. (v.Name or i)]:SetDisable(v.Disable)
            self["Container_Element_" .. (v.Name or i)].OnSelect = v.Func
        elseif v.Type == "NB" then
            self["Container_Element_" .. (v.Name or i)] = Widgets:CreateSlider(self.Content)
            self["Container_Element_" .. (v.Name or i)]:Size(self.Content:GetWidth() - 2 * Padding - (v.HSpacing or 0), 60)
            self["Container_Element_" .. (v.Name or i)]:SetText(v.Text, 11, 1, 1, 1, "LEFT", "TOP")
            self["Container_Element_" .. (v.Name or i)]:SetDisable(v.Disable)
            self["Container_Element_" .. (v.Name or i)]:SetMinStepMax(v.Min, v.Step, v.Max)
            self["Container_Element_" .. (v.Name or i)]:SetValue(v.Value)
            self["Container_Element_" .. (v.Name or i)].IgnoreSlideUpdates = v.IgnoreSlideUpdates
            self["Container_Element_" .. (v.Name or i)]:SetPoint("CENTER")
            self["Container_Element_" .. (v.Name or i)].OnValueChanged = v.Func
        elseif v.Type == "CP" then 
            self["Container_Element_" .. (v.Name or i)] = Widgets:CreateColorSelect(self.Content)
            self["Container_Element_" .. (v.Name or i)]:Size(self.Content:GetWidth() - 2 * Padding - (v.HSpacing or 0), 20)
            self["Container_Element_" .. (v.Name or i)]:SetText(v.Text, 11, 1, 1, 1, "LEFT", "TOP")
            self["Container_Element_" .. (v.Name or i)]:SetDisable(v.Disable)
            self["Container_Element_" .. (v.Name or i)]:SetValue(v.r,v.g,v.b,v.a)
            self["Container_Element_" .. (v.Name or i)].OnValueChanged = v.Func    
        elseif v.Type == "TB" then
            self["Container_Element_" .. (v.Name or i)] = Widgets:CreateTextBox(self.Content, v.Lines)
            self["Container_Element_" .. (v.Name or i)]:Size(self.Content:GetWidth() - 2 * Padding - (v.HSpacing or 0), 60 + (type(v.Lines) == "number" and (v.Lines - 1) * 11 or 0))
            self["Container_Element_" .. (v.Name or i)]:SetText(v.Text, 11, 1, 1, 1, "LEFT", "TOP")
            self["Container_Element_" .. (v.Name or i)]:SetValue(v.Value)
            self["Container_Element_" .. (v.Name or i)]:SetDisable(v.Disable)
            self["Container_Element_" .. (v.Name or i)].OnValueChanged = v.Func
        elseif v.Type == "SL" then
            self["Container_Element_" .. (v.Name or i)] = Widgets:CreateSelectList(self.Content)
            self["Container_Element_" .. (v.Name or i)]:Size(self.Content:GetWidth() - 2 * Padding - (v.HSpacing or 0), v.Height)
            self["Container_Element_" .. (v.Name or i)]:SetText(v.Text, 11, 1, 1, 1, "LEFT", "TOP")
            self["Container_Element_" .. (v.Name or i)]:AddItems(v.List)
            self["Container_Element_" .. (v.Name or i)]:SetDisable(v.Disable)
            self["Container_Element_" .. (v.Name or i)]:SelectItem(v.Selected)
            self["Container_Element_" .. (v.Name or i)].OnSelect = v.Func
        end

        if self["Container_Element_" .. (v.Name or i)] then
            insert(self.LM, "Container_Element_" .. (v.Name or i))
            if i > 1 then
                self["Container_Element_" .. (v.Name or i)]:SetPoint("TOPLEFT", self["Container_Element_" .. PrevElement], "BOTTOMLEFT", (v.HSpacing or 0) - PrevIndent, -Padding - (v.VSpacing or 0))
            else
                self["Container_Element_" .. (v.Name or i)]:SetPoint("TOPLEFT", Padding + (v.HSpacing or 0) - PrevIndent, -Padding - (v.VSpacing or 0))
            end
            if v.Type == "LB" then
                self["Container_Element_" .. (v.Name or i)]:GetHeight(self["Container_Element_" .. (v.Name or i)]:GetTop() - self["Container_Element_" .. (v.Name or i)]:GetBottom())
            end
            self["Container_Element_" .. (v.Name or i)].Tooltip = v.Tooltip;
            self["Container_Element_" .. (v.Name or i)].SpecialTooltip = v.SpecialTooltip;
            PrevIndent = v.HSpacing or 0
            Height = Height + self["Container_Element_" .. (v.Name or i)]:GetHeight() + Padding + (v.VSpacing or 0)
            PrevElement = (v.Name or i)
            i = i + 1
        end
    end
    self.Content:Size(nil, Height)

    table.wipe(Items)

    return Height + 2 * BorderSize
end

local function ChangeItem(self, Item, Par, Value)
    if self["Container_Element_" .. Item] then
        if Par == "Selected" and self["Container_Element_" .. Item].SetFocus then
            self["Container_Element_" .. Item]:SetFocus(Value)
        elseif Par == "Selected" and self["Container_Element_" .. Item].SelectItem then
            self["Container_Element_" .. Item]:SelectItem(Value)
        elseif Par == "Disable" and self["Container_Element_" .. Item].SetDisable then
            self["Container_Element_" .. Item]:SetDisable(Value)
        elseif Par == "Hide" then
            if Value then
                self["Container_Element_" .. Item]:Hide()
            else
                self["Container_Element_" .. Item]:Show()
            end
        elseif Par == "Value" and self["Container_Element_" .. Item].SetValue then
            self["Container_Element_" .. Item]:SetValue(Value)
        elseif Par == "Value" and self["Container_Element_" .. Item].SetChecked then
            self["Container_Element_" .. Item]:SetChecked(Value)
        elseif Par == "Value" and self["Container_Element_" .. Item].SetSelected then
            self["Container_Element_" .. Item]:SetSelected(Value)
        elseif Par == "List" and self["Container_Element_" .. Item].AddItems then
            self["Container_Element_" .. Item]:AddItems(Value)
        elseif Par == "Text" and self["Container_Element_" .. Item].SetText then
            self["Container_Element_" .. Item]:SetText(Value)
        end
    end
end

Widgets.CreateContainer = function(self, Parent)
    local Container = Widgets:CreateFrame(Parent)

    Container.ScrollFrame = Widgets:CreateScrollFrame(Container)
    Container.ScrollFrame:SetPoint("BOTTOMLEFT", BorderSize, BorderSize)
    Container.ScrollFrame:SetPoint("TOPRIGHT", -BorderSize, -BorderSize)

    Container:Backdrop()
    Container.Content = Widgets:CreateFrame(Container.ScrollFrame)
    Container.Content:SetPoint("TOPLEFT")
    Container.Content:Size(100, 1000)
    Container.ScrollFrame:SetScrollChild(Container.Content)

    Container.AddItems = AddItems
    Container.ClearItems = ReleaseElements
    Container.ChangeItem = ChangeItem

    Container:SetScript("OnMouseWheel",
        function(self, delta)
            self:SetScroll(-ScrollMulti * delta, true)
        end)

    Container:SizeChanged(function(self)
        self.Content:Size(self:GetWidth() - 2 * BorderSize, nil)
    end)

    Container.ScrollDistance = 0

    Container.SetScroll = function(self, Value, Increment)
        Value = Value or self.ScrollDistance
        if Increment then
            self.ScrollDistance = self.ScrollDistance + Value
        else
            self.ScrollDistance = Value
        end
        if self.ScrollDistance < 0 then
            self.ScrollDistance = 0
        elseif self.ScrollDistance > self.Content:GetHeight() - self.ScrollFrame:GetHeight() then
            if self.Content:GetHeight() - self.ScrollFrame:GetHeight() > 0 then
                self.ScrollDistance = self.Content:GetHeight() - self.ScrollFrame:GetHeight()
            else
                self.ScrollDistance = 0
            end
        end
        self.ScrollFrame:SetVerticalScroll(self.ScrollDistance)
        --self.ScrollBar.Bar:SetPoint("TOP",self.ScrollBar.Up,"BOTTOM",0,-self.ScrollDistance/self.ScrollFrame.content:GetHeight()*(self.ScrollBar:GetHeight() - self.ScrollBar.Up:GetHeight() - self.ScrollBar.Down:GetHeight()))
    end

    insert(Container.LM, "ScrollFrame")
    insert(Container.LM, "Content")
    insert(Container.LM, "ClearItems")
    insert(Container.LM, "AddItems")
    insert(Container.LM, "ChangeItem")
    insert(Container.LM, "ScrollDistance")
    insert(Container.LM, "SetScroll")
    insert(Container.LM, "NumberOfItems")
    insert(Container.LM, "PrevItem")
    insert(Container.LM, "PrevIndent")
    insert(Container.LM, "ContainerHeight")

    return Container
end
