local Widgets = _Avyr_GUI_Widgets_Lib
local pi, sin, cos, ipairs, wipe, select, insert, UIParent, CreateFrame, InCombatLockdown, GetCursorPosition, sqrt = math.pi, math.sin, math.cos, ipairs, table.wipe, select, table.insert, UIParent, CreateFrame, InCombatLockdown, GetCursorPosition, math.sqrt;
local pairs, type = pairs, type;
local BorderSize, Padding = Widgets.BorderSize, Widgets.Padding;
local _Avyr_GUI_Widgets_Tooltip = _Avyr_GUI_Widgets_Tooltip;

local function CornerRotationCalc(deg)
    local rads = deg / 180 * pi;
    local x = 0.5 + cos(rads) / sqrt(2)
    local y = 0.5 - sin(rads) / sqrt(2)
    return x, y
end

local Frames = {}

local function Release(self)
    
    --[[
    for i, v in ipairs(self.LM) do
        if type(self[v]) == "table" and self[v].Release then
            self[v]:Release();
        end
        self[v] = nil
    end
    wipe(self.LM);
]]
    for i,v in pairs(self) do 
        if type(v) == "table" and v.Release then
            v:Release();
        end
        v = nil
    end 

    self:Hide();

    --[[
        for i, v in ipairs(self.OnReleaseFuncs) do
            v(self)
        end
        wipe(self.OnReleaseFuncs)]]
    wipe(self.OnSizeChangedFuncs)
    self:Backdrop("Clear")
    for i = 1, 20 do
        if self["Text" .. i] then
            self["Text" .. i]:Release()
            self["Text" .. i] = nil
        end
    end
    if self.Icon then
        self.Icon:Release()
        self.Icon = nil
    end
    self:SetFrameStrata("MEDIUM")
    self:EnableMouse(false)
    self:SetDisable()
    self:SetScale(1)
    self:SetClampedToScreen(false)
    self:SetScript("OnMouseWheel", nil)
    self:SetScript("OnEnter", nil)
    self:SetScript("OnLeave", nil)
    self:SetScript("OnMouseDown", nil)
    self:SetScript("OnMouseUp", nil)
    self:SetScript("OnUpdate", nil)
    self:EnableMouse(false)
    self.OnMove = nil
    self.CheckCombatLock = nil
    self.PosParameters = nil
    self.MouseInteraction = nil
    self.EnterLeave = nil
    self.Borderless = nil;
end

local function SetDisable(self, Value)
    if Value then
        self:SetAlpha(0.4)
    else
        self:SetAlpha(1)
    end
    self.Disable = Value
end

local function SetText(self, ...)
    local firstValue = ...
    if type(firstValue) == "table" then 
            for ri = 1, 20 do
                if self["Text" .. ri] then
                    self["Text" .. ri]:Release()
                    self["Text" .. ri] = nil
                else
                    break
                end
            end

            local Height = 2*BorderSize
            for i,v in ipairs(...) do
                if not self["Text" .. i] then
                    self["Text" .. i] = Widgets:FetchFontString(self)
                end
                if i > 1 then 
                    self["Text" .. i]:SetPoint("TOPLEFT",self["Text" .. i - 1],"BOTTOMLEFT", 0, -Padding)
                    self["Text" .. i]:SetPoint("TOPRIGHT",self["Text" .. i - 1],"BOTTOMRIGHT", 0, -Padding)    
                    Height = Height + Padding
                else 
                    self["Text" .. i]:SetPoint("TOPLEFT", BorderSize + Padding, -BorderSize - Padding)
                    self["Text" .. i]:SetPoint("TOPRIGHT", -BorderSize - Padding, -BorderSize - Padding)
                    Height = Height + 2*Padding
                end  
                self["Text" .. i]:SetText(v.Text or "")
                if v.r and v.g and v.b then
                    self["Text" .. i]:SetTextColor(v.r, v.g, v.b)
                end
                if v.FontSize then
                    local Font, _, Flags = self["Text" .. i]:GetFont()
                    self["Text" .. i]:SetFont(Font, v.FontSize, Flags)
                    Height = Height + v.FontSize
                else 
                    Height = Height + 11
                end
                if v.HAlign then
                    self["Text" .. i]:SetJustifyH(v.HAlign)
                end
                if v.VAlign then
                    self["Text" .. i]:SetJustifyV(v.VAlign)
                end
                self["Text" .. i]:SetWordWrap(v.Wrap)
            end
            self:Size(nil,Height)
    else 
        for i = 1, 20 do
            if not select((1 + 7 * (i - 1)), ...) then
                for ri = i, 20 do
                    if self["Text" .. ri] then
                        self["Text" .. ri]:Release()
                        self["Text" .. ri] = nil
                    else
                        break
                    end
                end
                break
            end
            local Text, FontSize, r, g, b, HAlign, VAlign = select((1 + 7 * (i - 1)), ...)
            if not self["Text" .. i] then
                self["Text" .. i] = Widgets:FetchFontString(self)
                self["Text" .. i]:SetPoint("BOTTOMLEFT", BorderSize + Padding, BorderSize + Padding)
                self["Text" .. i]:SetPoint("TOPRIGHT", -BorderSize - Padding, -BorderSize - Padding)
            end
            self["Text" .. i]:SetText(Text or "")
            if r and g and b then
                self["Text" .. i]:SetTextColor(r, g, b)
            end
            if FontSize then
                local Font, _, Flags = self["Text" .. i]:GetFont()
                self["Text" .. i]:SetFont(Font, FontSize, Flags)
            end
            if HAlign then
                self["Text" .. i]:SetJustifyH(HAlign)
            end
            if VAlign then
                self["Text" .. i]:SetJustifyV(VAlign)
            end
        end
    end 
end 

local function SetIcon(self, TexturePath, l, r, t, b, rC, gC, bC, BorderGap, Rotation, LockAspectRatio)
    if TexturePath then
        if not self.Icon then
            self.Icon = Widgets:FetchTexture(self)
        end
        self.Icon:ClearAllPoints();
        local Gap = BorderSize
        if BorderGap then
            Gap = Gap + Padding
        end
        if LockAspectRatio then
            if type(LockAspectRatio) == "boolean" then
                self.Icon:SetPoint("CENTER")
                local MinSize = self:GetHeight() < self:GetWidth() and self:GetHeight() or self:GetWidth()
                self.Icon:SetSize(MinSize - 2 * Gap, MinSize - 2 * Gap)
            elseif LockAspectRatio == "LEFT" then
                self.Icon:SetPoint("BOTTOMLEFT", Gap, Gap)
                self.Icon:SetPoint("TOPRIGHT", self, "TOPLEFT", (self:GetHeight() - Gap), -Gap)
            end
        else
            self.Icon:SetPoint("BOTTOMLEFT", Gap, Gap)
            self.Icon:SetPoint("TOPRIGHT", -Gap, -Gap)
        end
        self.Icon:SetTexture(TexturePath)
        if Rotation then
            local TLx, TLy = CornerRotationCalc(Rotation + 225)
            local BLx, BLy = CornerRotationCalc(Rotation + 135)
            local TRx, TRy = CornerRotationCalc(Rotation - 45)
            local BRx, BRy = CornerRotationCalc(Rotation + 45)
            l = l or 0;
            r = r or 1; r = 1 - r;
            t = t or 0;
            b = b or 1; b = 1 - b;
            --print(TLx + l, TLy - t, BLx + l, BLy + b, TRx - r, TRy - t, BRx - r, BRy + b)
            --self.Icon:SetTexCoord(TLx + l, TLy - t, BLx + l, BLy + b, TRx - r, TRy - t, BRx - r, BRy + b)
            self.Icon:SetTexCoord(TLx, TLy, BLx, BLy, TRx, TRy, BRx, BRy)
        else
            self.Icon:SetTexCoord(l or 0, r or 1, t or 0, b or 1)
        end
        self.Icon:SetVertexColor(rC or 1, gC or 1, bC or 1, 1)
    else
        if self.Icon then
            self.Icon:Release()
            self.Icon = nil
        end
    end
end

local function SetSize(self, w, h)
    if w then
        self:SetWidth(w)
    end
    if h then
        self:SetHeight(h)
    end
    for i, v in ipairs(self.OnSizeChangedFuncs) do --at time the inbuilt OnSizeChanged event does not correctly trigger
        v(self)
    end
end

local function PresetBorders(self, State, Focus)
    State = State or "Normal"
    local r2, g2, b2, a2 = Widgets.BorderColor.r, Widgets.BorderColor.g, Widgets.BorderColor.b, Widgets.BorderColor.a;
    if Focus then
        r2, g2, b2, a2 = Widgets.HighlightColor.r, Widgets.HighlightColor.g, Widgets.HighlightColor.b, Widgets.HighlightColor.a;
    elseif self.Borderless then
        r2, g2, b2, a2 = Widgets.BackdropColor.r, Widgets.BackdropColor.g, Widgets.BackdropColor.b, 0;
    end
    if State == "Pushed" then
        self:Backdrop("Border", Widgets.HighlightColor.r, Widgets.HighlightColor.g, Widgets.HighlightColor.b, r2, g2, b2, Widgets.HighlightColor.a, a2, true)
    elseif State == "MouseOver" then
        self:Backdrop("Border", Widgets.HighlightColor.r / 2, Widgets.HighlightColor.g / 2, Widgets.HighlightColor.b / 2, r2, g2, b2, Widgets.HighlightColor.a, a2, true)
    elseif self.Borderless then
        self:Backdrop("Border", Widgets.BackdropColor.r, Widgets.BackdropColor.g, Widgets.BackdropColor.b, r2, g2, b2, 0, a2, true)
    else
        self:Backdrop("Border", Widgets.BorderColor.r, Widgets.BorderColor.g, Widgets.BorderColor.b, r2, g2, b2, Widgets.BorderColor.a, a2, true)
    end
end

local function SetBorderless(self, Borderless, r, g, b, a)
    self.Borderless = Borderless
    self:Backdrop("Backdrop",r,g,b,nil,nil,nil,a);
end

local function Backdrop(self, Type, r, g, b, r2, g2, b2, a, a2, BorderlessOverwrite)
    if Type and Type == "Clear" then 
        if self.BorderL then
            self.BorderL:Release()
            self.BorderL = nil
        end 
        if self.BorderT then
            self.BorderT:Release()
            self.BorderT = nil
        end 
        if self.BorderR then
            self.BorderR:Release()
            self.BorderR = nil
        end 
        if self.BorderB then
            self.BorderB:Release()
            self.BorderB = nil
        end 
        if self.BackdropTexture then
            self.BackdropTexture:Release()
            self.BackdropTexture = nil
        end 
        return nil
    end
    if self.Borderless then
        if self.BorderL then
            self.BorderL:Release()
            self.BorderL = nil
        end 
        if self.BorderT then
            self.BorderT:Release()
            self.BorderT = nil
        end 
        if self.BorderR then
            self.BorderR:Release()
            self.BorderR = nil
        end 
        if self.BorderB then
            self.BorderB:Release()
            self.BorderB = nil
        end 
    end
    if not Type or Type == "Backdrop" then
        if not self.BackdropTexture then
            self.BackdropTexture = Widgets:FetchTexture(self, "BACKGROUND")
        end
        if self.Borderless then 
            self.BackdropTexture:SetPoint("BOTTOMLEFT", 0, 0)
            self.BackdropTexture:SetPoint("TOPRIGHT", 0, 0)
        else 
            self.BackdropTexture:SetPoint("BOTTOMLEFT", BorderSize, BorderSize)
            self.BackdropTexture:SetPoint("TOPRIGHT", -BorderSize, -BorderSize)
        end 
        if r2 and g2 and b2 then 
            self.BackdropTexture:SetColorTexture(1, 1, 1, 1)
            self.BackdropTexture:SetGradientAlpha("HORIZONTAL", r, g, b, a, r2, g2, b2, a2)
        else 
            self.BackdropTexture:SetColorTexture(r or Widgets.BackdropColor.r, g or Widgets.BackdropColor.g, b or Widgets.BackdropColor.b, a or Widgets.BackdropColor.a)
        end 
    end
    if (not self.Borderless or BorderlessOverwrite) and (not Type or Type == "Border") then
        r = r or Widgets.BorderColor.r
        b = b or Widgets.BorderColor.b
        g = g or Widgets.BorderColor.g
        a = a or Widgets.BorderColor.a
        r2 = r2 or Widgets.BorderColor.r
        b2 = b2 or Widgets.BorderColor.b
        g2 = g2 or Widgets.BorderColor.g
        a2 = a2 or Widgets.BorderColor.a
        if not self.BorderL then
            self.BorderL = Widgets:FetchTexture(self, "BORDER")
            self.BorderL:SetPoint("TOPLEFT")
            self.BorderL:SetPoint("BOTTOMLEFT")
            self.BorderL:SetWidth(BorderSize)
        end
        self.BorderL:SetColorTexture(r, g, b, a)

        if not self.BorderR then
            self.BorderR = Widgets:FetchTexture(self, "BORDER")
            self.BorderR:SetPoint("TOPRIGHT")
            self.BorderR:SetPoint("BOTTOMRIGHT")
            self.BorderR:SetWidth(BorderSize)
        end
        self.BorderR:SetColorTexture(r2, g2, b2, a2)

        if not self.BorderT then
            self.BorderT = Widgets:FetchTexture(self, "BORDER")
            self.BorderT:SetPoint("TOPLEFT", self.BorderL, "TOPRIGHT")
            self.BorderT:SetPoint("TOPRIGHT", self.BorderR, "TOPLEFT")
            self.BorderT:SetHeight(BorderSize)
        end
        self.BorderT:SetColorTexture(1, 1, 1, 1)
        self.BorderT:SetGradientAlpha("HORIZONTAL", r, g, b, a, r2, g2, b2, a2)

        if not self.BorderB then
            self.BorderB = Widgets:FetchTexture(self, "BORDER")
            self.BorderB:SetPoint("BOTTOMLEFT", self.BorderL, "BOTTOMRIGHT")
            self.BorderB:SetPoint("BOTTOMRIGHT", self.BorderR, "BOTTOMLEFT")
            self.BorderB:SetHeight(BorderSize)
        end
        self.BorderB:SetColorTexture(1, 1, 1, 1)
        self.BorderB:SetGradientAlpha("HORIZONTAL", r, g, b, a, r2, g2, b2, a2)
    end
end

local function OnRelease(self, Func)
    insert(self.OnReleaseFuncs, Func)
end

local function SizeChanged(self, Func)
    insert(self.OnSizeChangedFuncs, Func)
end

local function StopMoving(self, Button)
    self:SetScript("OnUpdate", nil)
    if self.MouseInteraction and self.Interactable then
        self:MouseInteraction(true, Button)
    end
    if self.Movable and self.OnMove then
        self.OnMove(self)
    end
end

local function InternalMoveFunc_OnUpdate(self, elapsed)
    if self.CheckCombatLock and InCombatLockdown() then
        return StopMoving(self)
    end
    local MouseX, MouseY = GetCursorPosition()
    local Scale = self:GetEffectiveScale() 
    MouseX = MouseX / Scale
    MouseY = MouseY / Scale
    self:SetPoint(self.PosParameters.Point, self.PosParameters.Ref, self.PosParameters.RefPoint, MouseX + self.PosParameters.xDif, MouseY + self.PosParameters.yDif)
end

local function InternalMoveFunc_OnUpdate_WithFeedback(self, elapsed)
    if self.CheckCombatLock and InCombatLockdown() then
        return StopMoving(self)
    end
    local MouseX, MouseY = GetCursorPosition()
    local Scale = self:GetEffectiveScale()
    MouseX = MouseX / Scale
    MouseY = MouseY / Scale
    self:SetPoint(self.PosParameters.Point, self.PosParameters.Ref, self.PosParameters.RefPoint, MouseX + self.PosParameters.xDif, MouseY + self.PosParameters.yDif)
    self:MouseDownFeedback()
end

local function StartMoving(self, Button)
    if self.MouseInteraction and self.Interactable then
        self:MouseInteraction(false, Button)
    end
    if self.Movable then
        if self.CheckCombatLock and InCombatLockdown() then
            return nil
        end
        local Point, Ref, RefPoint, x, y = self:GetPoint(1)
        x = x or 0
        y = y or 0
        local MouseX, MouseY = GetCursorPosition()
        local Scale = self:GetEffectiveScale()
        self.PosParameters = { Point = Point, Ref = Ref, RefPoint = RefPoint, xDif = x - MouseX / Scale, yDif = y - MouseY / Scale }
        self:ClearAllPoints()
        if self.MouseDownFeedback then
            self:SetScript("OnUpdate", InternalMoveFunc_OnUpdate_WithFeedback)
        else
            self:SetScript("OnUpdate", InternalMoveFunc_OnUpdate)
        end
    elseif self.MouseDownFeedback then
        self:SetScript("OnUpdate", function(self) self:MouseDownFeedback() end)
    end
end

local function MakeMovable(self, Movable)
    if Movable and not self.Interactable then
        self:SetScript("OnMouseDown", StartMoving)
        self:SetScript("OnMouseUp", StopMoving)
    elseif not Movable then
        if not self.Interactable then
            self:SetScript("OnMouseDown", nil)
            self:SetScript("OnMouseUp", nil)
            self:EnableMouse(false)
        end
        self:SetScript("OnUpdate", nil)
    end
    self.Movable = Movable;
end

local function MakeInteractable(self, Inter)
    if Inter then
        if not self.Movable then
            self:SetScript("OnMouseDown", StartMoving)
            self:SetScript("OnMouseUp", StopMoving)
        end
        self:SetScript("OnLeave", function()
            _Avyr_GUI_Widgets_Tooltip:Hide();
            if self.EnterLeave then
                self:EnterLeave();
            end
        end)
        self:SetScript("OnEnter", function()
            if self.Tooltip and self.SpecialTooltip then
                _Avyr_GUI_Widgets_Tooltip:SetTT(self, self.Tooltip, nil, nil, self.SpecialTooltip);
            elseif self.Tooltip then
                _Avyr_GUI_Widgets_Tooltip:SetTT(self, self.Tooltip);
            end
            if self.EnterLeave then
                self:EnterLeave(true);
            end
        end)
    else
        self:SetScript("OnLeave")
        self:SetScript("OnEnter")
        if not self.Movable then
            self:SetScript("OnMouseDown", nil)
            self:SetScript("OnMouseUp", nil)
            self:EnableMouse(false)
        end
    end
    self.Interactable = Inter;
end

Widgets.CreateFrame = function(self, Parent)
    local Frame = 0
    for i, v in ipairs(Frames) do
        if v.Released then
            Frame = i
            break
        end
    end
    if Frame == 0 then
        Frame = 1
        insert(Frames, Frame, CreateFrame("Frame", nil, Parent or UIParent))
        Frames[Frame]:SetClampedToScreen(false)

        Frames[Frame].OnReleaseFuncs = {}
        Frames[Frame].OnSizeChangedFuncs = {}

        Frames[Frame].Release = Release
        Frames[Frame].Backdrop = Backdrop
        Frames[Frame].OnRelease = OnRelease
        Frames[Frame].SizeChanged = SizeChanged
        Frames[Frame].SetText = SetText
        Frames[Frame].SetIcon = SetIcon
        Frames[Frame].Size = SetSize
        Frames[Frame].SetDisable = SetDisable
        Frames[Frame].PresetBorders = PresetBorders
        Frames[Frame].MakeMovable = MakeMovable
        Frames[Frame].MakeInteractable = MakeInteractable
        Frames[Frame].SetBorderless = SetBorderless

        Frames[Frame].LM = {};
    end
    Frames[Frame]:SetParent(Parent or UIParent)
    Frames[Frame]:ClearAllPoints()
    Frames[Frame]:Show()
    Frames[Frame].Released = false

    return Frames[Frame]
end
