local Widgets = _Avyr_GUI_Widgets_Lib
local getn, floor, ceil, ipairs, type, insert, wipe = getn, floor, ceil, ipairs, type, table.insert, table.wipe;
local BorderSize, Padding = Widgets.BorderSize, Widgets.Padding;

local TranslationTable = {
    ["Search"] = {
        ["deDE"] = "Suchen",
    },
}
local Translation = function(Text)
    if type(Text) == "string" then 
        if TranslationTable[Text] then 
            local Lang = GetLocale()
            if TranslationTable[Text][Lang] then 
                return TranslationTable[Text][Lang]
            else 
                return Text
            end 
        else 
            return Text.."NT"
        end 
    end 
end 

Widgets.SearchList = function(self, Parent, Title)
    local List = self:CreateFrame(Parent)

    List.NoteListSearchLB = Widgets:FetchFontString(List)
    List.NoteListSearchLB:SetText(Translation("Search"))
    List.NoteListSearchLB:SetHeight(20)
    List.NoteListSearchLB:SetPoint("TOPLEFT")

    insert(List.LM,"NoteListSearchLB")

    List.NoteSearch = Widgets:CreateBasicTextBox(List,nil,"LEFT",nil,function(Value)
        if Value ~= self.FilterTerm then 
            List:SetFilter(Value)
        end 
    end)
    List.NoteSearch:Size(nil,20)
    List.NoteSearch:SetPoint("TOPLEFT", List.NoteListSearchLB, "TOPRIGHT",Padding,0)
    List.NoteSearch:SetPoint("TOPRIGHT")
    List.NoteSearch:SetValue("")

    insert(List.LM,"NoteSearch")


    List.Scroll = self:CreateAdancedScrollFrame(List)
    List.Scroll:SetPoint("TOPLEFT",List.NoteListSearchLB,"BOTTOMLEFT",0,-Padding)
    List.Scroll:SetPoint("BOTTOMRIGHT")


    List.ListOfItems = {}
    List.SelectedItem = nil

    List.AddItem = function(self, Text, Key, Icon)
        local Height = 8;
        for i,v in ipairs(Text) do 
            Height = Height + v.FontSize + 2;
        end 
        if getn(self.ListOfItems) == 0 then 
            List.Scroll.Content:Size(nil,Padding)
        end 

        insert(self.ListOfItems,{
            Key = Key,
            Text = Text, 
            Height = Height,
            Icon = Icon
        })
        self.Scroll.Content:Size(nil, Height + List.Scroll.Content:GetHeight() + Padding)

        self:Refresh()
    end 

    List.Scroll.OnScroll = function()
        List:Refresh()
    end 

    List.Refresh = function(self)
        local Offset = self.Scroll.ScrollOffset
        local DisplayHeight = self.Scroll:GetHeight() - 2*Padding
        local currentHeightOffset = Padding
        local DisplayIndex = 1
        for i,v in ipairs(self.ListOfItems) do 
            if type(v.Key) == "nil" or type(self.FilterTerm) == "nil" or v.Key:find(self.FilterTerm) then 
                if currentHeightOffset + v.Height + Padding >= Offset and Offset + DisplayHeight >= currentHeightOffset + Padding then 
                    if not self["DisplayButton_"..DisplayIndex] then 
                        self["DisplayButton_"..DisplayIndex] = Widgets:CreateButton(self.Scroll.Content)
                        if not tContains(List.LM,"DisplayButton_"..DisplayIndex ) then 
                            insert(List.LM, "DisplayButton_"..DisplayIndex)
                        end 
                        insert(self["DisplayButton_"..DisplayIndex].LM,"SearchListGlobalIndex")
                    end 
                    if v.Icon then 
                        self["DisplayButton_"..DisplayIndex]:SetIcon(v.Icon, nil, nil, nil, nil, nil, nil, nil, nil, nil, "LEFT")
                    end 
                    self["DisplayButton_"..DisplayIndex]:SetPoint("TOPLEFT",self.Scroll.Content,"TOPLEFT",Padding,-currentHeightOffset)
                    self["DisplayButton_"..DisplayIndex]:Size(self.Scroll.Content:GetWidth() - 2*Padding, nil)
                    self["DisplayButton_"..DisplayIndex]:SetText(v.Text)
                    self["DisplayButton_"..DisplayIndex].SearchListGlobalIndex = i;                
                    if self.SelectedItem and i == self.SelectedItem then 
                        self["DisplayButton_"..DisplayIndex]:SetFocus(true)
                    else 
                        self["DisplayButton_"..DisplayIndex]:SetFocus(false)
                    end 
                    self["DisplayButton_"..DisplayIndex].OnClick = function()
                        self:SelectItem(i)
                    end 
                    --self["DisplayButton_"..DisplayIndex]:
                    DisplayIndex = DisplayIndex + 1
                end 
                currentHeightOffset = currentHeightOffset + v.Height + Padding
            end 
        end 
        while (self["DisplayButton_"..DisplayIndex]) do 
            self["DisplayButton_"..DisplayIndex]:Release()
            self["DisplayButton_"..DisplayIndex] = nil
            DisplayIndex = DisplayIndex + 1
        end    
    end 

    List.SetFilter = function(self,Term)
        if Term ~= self.FilterTerm then 
            self.FilterTerm = Term or ""
            self:Refresh()
        end 
    end 

    List.SelectItem = function(self, Index)

        if type(Index) == "string" then 
            for i,v in ipairs(self.ListOfItems) do 
                if v.Key == Index then 
                    Index = i;
                end 
            end 
            if type(Index) == "string" then 
                return 
            end 
        elseif type(Index) == "number" then 
            if not self.ListOfItems[Index] then 
                Index = nil
            end 
        end 

        if self.SelectedItem ~= Index then 
            local DisplayIndex = 1;
            while (self["DisplayButton_"..DisplayIndex]) do 
                if Index and self["DisplayButton_"..DisplayIndex].SearchListGlobalIndex == Index then 
                    self["DisplayButton_"..DisplayIndex]:SetFocus(true)
                else 
                    self["DisplayButton_"..DisplayIndex]:SetFocus(false)
                end 
                DisplayIndex = DisplayIndex + 1
            end 

            self.SelectedItem = Index 
            if self.OnSelect then 
                self.OnSelect(self, Index)
            end
        end 
    end 

    List.Clear = function(self)
        self.SelectedItem = nil
        local DisplayIndex = 1
        while (self["DisplayButton_"..DisplayIndex]) do 
            self["DisplayButton_"..DisplayIndex]:Release()
            self["DisplayButton_"..DisplayIndex] = nil
            DisplayIndex = DisplayIndex + 1
        end     
        wipe(self.ListOfItems)
    end 

    List.EditItem = function(self, Index, Text, Key)
        if type(Index) == "string" then 
            for i,v in ipairs(self.ListOfItems) do 
                if v.Key == Index then 
                    Index = i
                end 
            end 
            if type(Index) == "nil" then 
                return 
            end 
        elseif type(Index) == "number" and not self.ListOfItems[Index] then 
            return 
        end 

        if type(Key) == "nil" and type(Text) == "nil" then 
            table.remove(self.ListOfItems,Index)
        else 
            self.ListOfItems[Index].Key = Key or self.ListOfItems[Index].Key
            self.ListOfItems[Index].Text = Text or self.ListOfItems[Index].Text

            local Height = 8;
            for i,v in ipairs(self.ListOfItems[Index].Text) do 
                Height = Height + v.FontSize + 2;
            end 

            self.ListOfItems[Index].Height = Height or self.ListOfItems[Index].Height
        end 
        self:Refresh()
    end 

    List.IsSelected = function(self, Index)
        return self.SelectedItem == Index
    end 

    insert(List.LM, "ListOfItems")
    insert(List.LM, "AddItem")
    insert(List.LM, "SelectItem")
    insert(List.LM, "Clear")
    insert(List.LM, "OnSelect")
    insert(List.LM, "EditItem")
    insert(List.LM, "SelectedItem")
    insert(List.LM, "Refresh")
    insert(List.LM, "IsSelected")
    insert(List.LM, "SetFilter")
    insert(List.LM, "FilterTerm")

    insert(List.LM, "AddItem")

    return List
end 