local Widgets = _Avyr_GUI_Widgets_Lib
local getn, floor, ceil, ipairs, type, insert = getn, floor, ceil, ipairs, type, table.insert;
local BorderSize, Padding = Widgets.BorderSize, Widgets.Padding;

local function Refresh(self, NewIndex, Increment)
    local Refresh = false
    if NewIndex and self.ListOfItems then
        if Increment then
            NewIndex = self.CurrentScroll + NewIndex
        end
        if NewIndex + self.ScrollFrame:GetHeight() > (getn(self.ListOfItems) * self.ItemOffset) + Padding then
            NewIndex = getn(self.ListOfItems) * self.ItemOffset - self.ScrollFrame:GetHeight() + Padding
        end
        if NewIndex < 0 then
            NewIndex = 0
        end
        if self.CurrentScroll ~= NewIndex then
            self.CurrentScroll = NewIndex
            Refresh = true
        end
    else
        Refresh = true
    end

    if Refresh then
        if self.ListOfItems then
            local OffsetIndex = floor(self.CurrentScroll / self.ItemOffset)
            if OffsetIndex < 0 then
                OffsetIndex = 0
            end
            if self.CurrentIndexOffset ~= OffsetIndex then
                for i = 1, self.LinesOnDisplay do
                    if self.ListOfItems[OffsetIndex + i] then
                        self.Content["Select_Element_" .. i]:Show()
                        if type(self.ListOfItems[OffsetIndex + i]) == "string" then 
                            self.Content["Select_Element_" .. i]:SetText(self.ListOfItems[OffsetIndex + i])
                            self.Content["Select_Element_" .. i].Tooltip = self.ListOfItems[OffsetIndex + i]
                        else 
                            self.Content["Select_Element_" .. i]:SetText(self.ListOfItems[OffsetIndex + i])
                        end 
                        if self.SelectedIndex == (OffsetIndex + i) then
                            self.Content["Select_Element_" .. i]:SetFocus(true)
                        else
                            self.Content["Select_Element_" .. i]:SetFocus(false)
                        end
                    else
                        self.Content["Select_Element_" .. i]:Hide()
                    end
                    self.CurrentIndexOffset = OffsetIndex
                end
            end
            self.SliderButton:SetPoint("TOP", self.SlidLine, "TOP", 0, -(Padding + (self.CurrentScroll / (self.ItemOffset + 0.00001)) * self.ScrollBarHeight / (getn(self.ListOfItems) + 0.00001)))
            self.ScrollFrame:SetVerticalScroll(self.CurrentScroll % self.ItemOffset)
        else
            for i = 1, self.LinesOnDisplay do
                self.Content["Select_Element_" .. i]:Hide()
            end
            self.SliderButton:SetPoint("TOP", self.SlidLine, "TOP", 0, -(Padding + self.ScrollBarHeight))
        end
    end
end

local function AddItems(self, Items)
    Items = Items or {}
    self.ListOfItems = Items
    local DisplayedRatio = (self:GetHeight() + self.TopOffset - 2 * BorderSize) / (getn(self.ListOfItems) * self.ItemOffset + Padding)
    if DisplayedRatio > 1 then
        DisplayedRatio = 1
    end
    self.SliderButton:SetHeight(self.ScrollBarHeight * DisplayedRatio)
    self.CurrentScroll = 0
    self.CurrentIndexOffset = nil --to force a full reset
    self:Refresh()
end

local function SelectItem(self, Index, UserInput, TextIndex)
    if type(Index) == "string" then
        if self.ListOfItems then 
            for i, v in ipairs(self.ListOfItems) do
                if (type(v) == "table" and v[TextIndex] == Index) or (type(v) == "string" and v == Index) then
                    Index = i;
                    break;
                end
            end
        end 
    end
    if type(Index) == "string" then 
        return --Matching problem
    end 
    if self.SelectedIndex ~= Index then
        if self.SelectedIndex then
            if self.SelectedIndex >= (1 + self.CurrentIndexOffset) and self.SelectedIndex <= (self.CurrentIndexOffset + self.LinesOnDisplay) then
                self.Content["Select_Element_" .. self.SelectedIndex - self.CurrentIndexOffset]:SetFocus(false)
            end
        end
        self.SelectedIndex = Index
        if self.SelectedIndex then
            if self.SelectedIndex >= (1 + self.CurrentIndexOffset) and self.SelectedIndex <= (self.CurrentIndexOffset + self.LinesOnDisplay) then
                self.Content["Select_Element_" .. self.SelectedIndex - self.CurrentIndexOffset]:SetFocus(true)
            end
        end
        if self.OnSelect and UserInput then
            self.OnSelect(self, self.SelectedIndex, self.ListOfItems[self.SelectedIndex])
        end
    end
end

Widgets.CreateSelectList = function(self, Parent,NoHeader, ItemSize)
    local Listbox = Widgets:CreateFrame(Parent)

    ItemSize = ItemSize or 20

    Listbox:SetScript("OnMouseWheel",
        function(_, delta)
            Listbox:Refresh(-4 * delta, true)
        end)

    Listbox.DownBtn = Widgets:CreateButton(Listbox)
    Listbox.DownBtn:SetPoint("BOTTOMRIGHT")
    Listbox.DownBtn:Size(20, 20)
    Listbox.DownBtn:SetIcon("Interface\\BUTTONS\\UI-MicroStream-Yellow.blp")
    Listbox.DownBtn.OnClick = function(self)
        Listbox:Refresh(Listbox.ItemOffset / 2, true)
    end

    Listbox.TopOffset = -20
    if NoHeader then 
        Listbox.TopOffset = 0
    end 

    Listbox.UpBtn = Widgets:CreateButton(Listbox)
    Listbox.UpBtn:SetPoint("TOPRIGHT", 0, Listbox.TopOffset)
    Listbox.UpBtn:Size(20, 20)
    Listbox.UpBtn:SetIcon("Interface\\BUTTONS\\UI-MicroStream-Yellow.blp", 1, 0, 1, 0)
    Listbox.UpBtn.OnClick = function(self)
        Listbox:Refresh(-Listbox.ItemOffset / 2, true)
    end

    Listbox.SlidLine = Widgets:FetchTexture(Listbox)
    Listbox.SlidLine:SetPoint("TOP", Listbox.UpBtn, "BOTTOM")
    Listbox.SlidLine:SetPoint("BOTTOM", Listbox.DownBtn, "TOP")
    Listbox.SlidLine:SetWidth(2)
    Listbox.SlidLine:SetColorTexture(1, 0.82, 0)

    Listbox.SliderButton = Widgets:CreateFrame(Listbox)
    Listbox.SliderButton:Size(20, 20)
    Listbox.SliderButton:SetPoint("TOP", Listbox.SlidLine, "TOP", 0, -Padding)
    Listbox.SliderButton:Backdrop()
    Listbox.SliderButton:Backdrop("Backdrop", 1, 0.82, 0)

    Listbox.ScrollFrameBackdrop = Widgets:CreateFrame(Listbox)
    Listbox.ScrollFrameBackdrop:Backdrop()
    Listbox.ScrollFrameBackdrop:SetPoint("BOTTOMLEFT")
    Listbox.ScrollFrameBackdrop:SetPoint("TOPRIGHT", Listbox.UpBtn, "TOPLEFT", -Padding, 0)

    Listbox.ScrollFrame = Widgets:CreateScrollFrame(Listbox.ScrollFrameBackdrop)
    Listbox.ScrollFrame:SetPoint("BOTTOMLEFT", Listbox.ScrollFrameBackdrop, BorderSize, BorderSize)
    Listbox.ScrollFrame:SetPoint("TOPRIGHT", Listbox.ScrollFrameBackdrop, -BorderSize, -BorderSize)

    --Listbox.ScrollFrame:Backdrop()
    Listbox.Content = Widgets:CreateFrame(Listbox.ScrollFrame)
    Listbox.Content:SetPoint("TOPLEFT")
    Listbox.Content:Size(100, 100)
    Listbox.Content:SetClampedToScreen(false)
    Listbox.ScrollFrame:SetScrollChild(Listbox.Content)

    Listbox.ItemOffset = 10

    Listbox:SizeChanged(function()
        local MainHeight = Listbox:GetHeight()
        Listbox.LinesOnDisplay = ceil((MainHeight - Padding + 20 + Listbox.TopOffset) / (ItemSize + Padding))
        --print(Listbox.LinesOnDisplay)
        Listbox.Content:Size(Listbox:GetWidth() - 20 - 2 * BorderSize - Padding, Listbox.LinesOnDisplay*(ItemSize + Padding) + Padding)
        Listbox.ItemOffset = 20 + Padding
        Listbox.ScrollBarHeight = MainHeight - 60 - 2 * Padding + 20 + Listbox.TopOffset
        for i = 1, Listbox.LinesOnDisplay do
            if not Listbox.Content["Select_Element_" .. i] then
                Listbox.Content["Select_Element_" .. i] = Widgets:CreateButton(Listbox.Content)
                insert(Listbox.Content.LM, "Select_Element_" .. i)

                Listbox.Content["Select_Element_" .. i]:SetText("", 11, 1, 1, 1, "LEFT")
                Listbox.Content["Select_Element_" .. i].ActiveCheck = function()
                    return not Listbox.Disable
                end
                Listbox.Content["Select_Element_" .. i].OnClick = function()
                    Listbox:SelectItem(i + floor(Listbox.CurrentScroll / Listbox.ItemOffset), true)
                end
                if Listbox.Content["Select_Element_" .. i] then
                    if i > 1 then
                        Listbox.Content["Select_Element_" .. i]:SetPoint("TOPLEFT",
                            Listbox.Content["Select_Element_" .. i - 1], "BOTTOMLEFT", 0, -Padding)
                    else
                        Listbox.Content["Select_Element_" .. i]:SetPoint("TOPLEFT", Padding, -Padding)
                    end
                end
            end
            Listbox.Content["Select_Element_" .. i]:Size(Listbox.Content:GetWidth() - 2 * Padding, ItemSize)
        end
    end)

    Listbox.CurrentScroll = 0

    Listbox.AddItems = AddItems
    Listbox.Refresh = Refresh
    Listbox.SelectItem = SelectItem

    insert(Listbox.LM, "SelectedIndex")
    insert(Listbox.LM, "CurrentIndexOffset")
    insert(Listbox.LM, "CurrentScroll")
    insert(Listbox.LM, "LinesOnDisplay")
    insert(Listbox.LM, "ItemOffset")
    insert(Listbox.LM, "ListOfItems")
    insert(Listbox.LM, "Content")
    insert(Listbox.LM, "ScrollFrame")
    insert(Listbox.LM, "ScrollFrameBackdrop")
    insert(Listbox.LM, "DownBtn")
    insert(Listbox.LM, "AddItems")
    insert(Listbox.LM, "Refresh")
    insert(Listbox.LM, "UpBtn")
    insert(Listbox.LM, "SlidLine")
    insert(Listbox.LM, "SliderButton")
    insert(Listbox.LM, "ScrollBarHeight")
    insert(Listbox.LM, "SelectItem")
    insert(Listbox.LM, "OnSelect")
    insert(Listbox.LM, "TopOffset")

    return Listbox
end
